package jalview.rest;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jalview.bin.Cache;
import jalview.rest.RestHandler.EndpointI;

public abstract class Endpoint implements EndpointI
{
  private String name = null;

  private API api = null;

  public void setName(String name)
  {
    this.name = name;
  }

  public String getName()
  {
    return this.name;
  }

  protected API getAPI()
  {
    return this.api;
  }

  public abstract void processEndpoint(HttpServletRequest request,
          HttpServletResponse response);

  /*
   * Shared methods below here
   */

  protected String[] getEndpointPathParameters(HttpServletRequest request)
  {
    String pathInfo = request.getPathInfo();
    int slashpos = pathInfo.indexOf('/', 1);
    return slashpos < 1 ? null
            : pathInfo.substring(slashpos + 1).split("/");
  }

  protected void returnError(HttpServletRequest request,
          HttpServletResponse response, String message)
  {
    response.setStatus(500); // set this to something better
    String endpointName = getName();
    Cache.error(getAPI().getName() + " error: endpoint " + endpointName
            + " failed: '" + message + "'");
    try
    {
      PrintWriter writer = response.getWriter();
      writer.write("Endpoint " + endpointName + ": " + message);
      writer.close();
    } catch (IOException e)
    {
      Cache.debug(e);
    }
  }

  protected String getRequestBody(HttpServletRequest request)
          throws IOException
  {
    StringBuilder sb = new StringBuilder();
    BufferedReader reader = request.getReader();
    try
    {
      String line;
      while ((line = reader.readLine()) != null)
      {
        sb.append(line).append('\n');
      }
    } finally
    {
      reader.close();
    }
    return sb.toString();
  }

}